/***********************************************************/
/*                                                         */
/* Amiga C Encyclopedia (ACE)           Amiga C Club (ACC) */
/* --------------------------           ------------------ */
/*                                                         */
/* Manual:  AmigaDOS                    Amiga C Club       */
/* Chapter: Handlers                    Tulevagen 22       */
/* File:    Example5B.c                 181 41  LIDINGO    */
/* Author:  Anders Bjerin               SWEDEN             */
/* Date:    93-03-16                                       */
/* Version: 1.3                                            */
/*                                                         */
/*   Copyright 1993, Anders Bjerin - Amiga C Club (ACC)    */
/*                                                         */
/* Registered members may use this program freely in their */
/*     own commercial/noncommercial programs/articles.     */
/*                                                         */
/***********************************************************/

/* This example demonstrates how you can use the Pipe handler */
/* to copy (pipe) data to another program. This is the second */
/* part of the example, program B. See example 5A for more    */
/* information.                                               */



/* Include the dos library definitions: */
#include <dos/dos.h>

/* Now we include the necessary function prototype files:         */
#include <clib/dos_protos.h>       /* General dos functions...    */
#include <clib/exec_protos.h>      /* System functions...         */
#include <stdio.h>                 /* Std functions [printf()...] */
#include <stdlib.h>                /* Std functions [exit()...]   */
#include <string.h>                /* Std functions [strlen()...] */



/* The maximum number of characters that we can store in */
/* our small buffer (including a NULL sign at the end):  */
#define MAX_LENGTH 512



/* Set name and version number: */
UBYTE *version = "$VER: AmigaDOS/Handlers/Example5B 1.3";



/* Declared our own function(s): */

/* Our main function: */
int main( int argc, char *argv[] );

/* Writes text to an already open file: */
/* (e.g. File, Console or Pipe handler) */
int print_text
(
  BPTR file,
  STRPTR text
);

/* Collects text from an already open file:*/
/* (e.g. File, Console or Pipe handler)    */
int collect_text
(
  BPTR file,
  STRPTR text
);



/* Main function: */

int main( int argc, char *argv[] )
{
  /* Store the number of characters used here: */
  int number;

  /* Create a buffers: */
  UBYTE user_input[ MAX_LENGTH ];

  /* A "BCPL" pointer to our Console window: */
  BPTR my_console;

  /* A "BCPL" pointer to our Pipe handler: */
  BPTR my_pipe;
  


  /* Open a Console window: */
  my_console = 
   Open( "CON:0/100/640/100/Program B/CLOSE/WAIT", MODE_NEWFILE );
  
  /* Have we opened the Console successfully? */
  if( my_console == NULL )
  {
    /* Inform the user: */
    printf( "Error! Could not open the Console device!\n" );

    /* Exit with an error code: */
    exit( 20 );
  }



  /* Open the Pipe handler: (We call the pipe "UserData", and  */
  /* we open the pipe handler as an old handler. Program A     */
  /* opens the handler as a new handler. Note that one program */
  /* must open the handler as new and the other program open   */
  /* the handler as old! The order does not matter.)           */
  my_pipe = 
   Open( "PIPE:UserData", MODE_OLDFILE );

  /* Have we opened the Pipe handler successfully? */
  if( my_pipe == NULL )
  {
    /* Inform the user: */
    printf( "Error! Could not open the Pipe handler!\n" );

    /* Note that since we have set the flag "WAIT" the Console */
    /* window will not be closed when we close the "file". The */
    /* user has to close it by him/her self. It is therefore   */
    /* best to give the user some instructions in the Console  */
    /* window itself:                                          */
    print_text( my_console, "Error! Could not open the Pipe handler!\n" );
    print_text( my_console, "Please close this window.\n\n" );
    
    /* Close the Console window: */
    Close( my_console );

    /* Exit with an error code: */
    exit( 21 );
  }



  /* Tell the user what will happen: */
  print_text( my_console,
    "Start Example7A and type in some text. The text you enter will\n" );
  print_text( my_console,
    "be copied (piped) to this window when you press Enter.\n" );
  print_text( my_console,
    "Note that you can not close this window until some text\n" );
  print_text( my_console,
    "has arrived at the pipe \"PIPE:UserData\" handler.\n\nPiped: " );

  /* Collect some text from the Pipe handler: */
  number = collect_text( my_pipe, user_input );

  /* Send the data we collected to the Console window: */
  print_text( my_console, user_input );

  /* All text has now been piped: */
  print_text( my_console,
    "\nNo oil, but the text was at least piped!\n" );
  print_text( my_console,
    "Please close the window.\n" );




  /* Close the Pipe handler: */
  Close( my_pipe );

  /* Close the consle window: */
  Close( my_console );

  /* Note! Since we have opened a Console window with a close */
  /* gadget and set the flag "WAIT" the Console window will   */
  /* actually first be closed when the user clicks on the     */
  /* close gadget or types "CTRL-\".                          */



  /* The End! */
  exit( 0 );
}



/* Writes text to an already opened file, and returns */
/* the number of characters actualy written.          */

int print_text
(
  BPTR file,
  STRPTR text
)
{
  /* Store the number of characters (bytes) actualy written here: */
  int characters_written;



  /* Write the text: */
  characters_written = Write( file, text, strlen( text ) );

  /* Returns the number of characters actually written: */
  return( TRUE );
}



/* Collects text from an already opened file, and returns */
/* the number of characters collected.                    */

int collect_text
(
  BPTR file,
  STRPTR text
)
{
  /* Store the number of characters (bytes) actualy read here: */
  int characters_read;



  /* Collect some text: */
  characters_read = Read( file, text, MAX_LENGTH - 1 );

  /* Put the NULL ('\0') sign at the end of the string: */
  text[ characters_read ] = NULL;

  /* Returns the number of characters collected: */
  return( characters_read );
}

